@@ -1,5 +1,6 @@
 from interfaces import *
 from api import *
+from api import _
 import declarative
 
 __all__ = ['Schema']
@@ -27,6 +28,11 @@ class Schema(FancyValidator):
 
         class MySubSchema(MySchema):
             name = None
+
+    Note that missing fields are handled at the Schema level.  Missing
+    fields can have the 'missing' message set to specify the error
+    message, or if that does not exist the *schema* message
+    'missingValue' is used.
     """
 
     # These validators will be applied before this schema:
@@ -52,8 +58,9 @@ class Schema(FancyValidator):
     order = []
 
     messages = {
-        'notExpected': 'The input field %(name)s was not expected.',
-        'missingValue': "Missing value",
+        'notExpected': _('The input field %(name)s was not expected.'),
+        'missingValue': _("Missing value"),
+        'badDictType': _("The input must be dict-like (not a %(type)s: %(value)r)"),
         }
 
     __mutableattributes__ = ('fields', 'chained_validators',
@@ -98,12 +105,26 @@ class Schema(FancyValidator):
         for name, value in self.fields.items():
             self.add_field(name, value)
     
+    def assert_dict(self, value, state):
+        """
+        Helper to assure we have proper input
+        """
+        if not hasattr(value, 'items'):
+            # Not a dict or dict-like object
+            raise Invalid(self.message('badDictType', state, type=type(value), value=value),
+                          value, state)
+            
     def _to_python(self, value_dict, state):
-        if not value_dict and self.if_empty is not NoDefault:
-            return self.if_empty
+        if not value_dict:
+            if self.if_empty is not NoDefault:
+                return self.if_empty
+            else:
+                value_dict = {}
 
         for validator in self.pre_validators:
             value_dict = validator.to_python(value_dict, state)
+
+        self.assert_dict(value_dict, state)
         
         new = {}
         errors = {}
@@ -143,9 +164,11 @@ class Schema(FancyValidator):
                     if self.ignore_key_missing:
                         continue
                     if self.if_key_missing is NoDefault:
-                        errors[name] = Invalid(
-                            self.message('missingValue', state),
-                            None, state)
+                        try:
+                            message = validator.message('missing', state)
+                        except KeyError:
+                            message = self.message('missingValue', state)
+                        errors[name] = Invalid(message, None, state)
                     else:
                         try:
                             new[name] = validator.to_python(self.if_key_missing, state)
@@ -192,6 +215,7 @@ class Schema(FancyValidator):
             __traceback_info__ = 'for_python chained_validator %s (finished %s)' % (validator, ', '.join(map(repr, finished)) or 'none')
             finished.append(validator)
             value_dict = validator.from_python(value_dict, state)
+        self.assert_dict(value_dict, state)
         new = {}
         errors = {}
         unused = self.fields.keys()
@@ -287,6 +311,13 @@ class Schema(FancyValidator):
         result.extend(self.fields.values())
         return result
 
+    def is_empty(self, value):
+        ## Generally nothing is empty for us
+        return False
+
+    def empty_value(self, value):
+        return {}
+
 def format_compound_error(v, indent=0):
     if isinstance(v, Exception):
         try:
