@@ -25,11 +25,9 @@ Validator/Converters for use with FormEncode.
 
 import re
 DateTime = None
-mxlookup = None
 httplib = None
 urlparse = None
-socket = None
-DNSError = None
+import socket
 from interfaces import *
 from api import *
 sha = random = None
@@ -42,6 +40,13 @@ import cgi
 
 import fieldstorage
 
+try:
+    import DNS
+    DNS.DiscoverNameServers()
+    have_dns=True
+except ImportError:
+    have_dns=False
+
 True, False = (1==1), (0==1)
 
 def _(s): return s # dummy translation function, nothing is translated here.
@@ -813,14 +818,22 @@ class DateValidator(FancyValidator):
         }
 
     def validate_python(self, value, state):
+        date_format = self.message('date_format', state)
+        if isinstance(date_format, unicode):
+            # strftime doesn't like unicode
+            encoding = 'utf8'
+            date_format = date_format.encode(encoding)
+        else:
+            encoding = None
         if self.earliest_date:
             if callable(self.earliest_date):
                 earliest_date = self.earliest_date()
             else:
                 earliest_date = self.earliest_date
             if value < earliest_date:
-                date_formatted = earliest_date.strftime(
-                    self.message('date_format', state))
+                date_formatted = earliest_date.strftime(date_format)
+                if encoding:
+                    date_formatted = date_formatted.decode(encoding)
                 raise Invalid(
                     self.message('after', state,
                                  date=date_formatted),
@@ -831,8 +844,9 @@ class DateValidator(FancyValidator):
             else:
                 latest_date = self.latest_date
             if value > latest_date:
-                date_formatted = latest_date.strftime(
-                    self.message('date_format', state))
+                date_formatted = latest_date.strftime(date_format)
+                if encoding:
+                    date_formatted = date_formatted.decode(encoding)
                 raise Invalid(
                     self.message('before', state,
                                  date=date_formatted),
@@ -841,8 +855,9 @@ class DateValidator(FancyValidator):
             dt_mod = import_datetime(self.datetime_module)
             now = datetime_now(dt_mod)
             if value < now:
-                date_formatted = now.strftime(
-                    self.message('date_format', state))
+                date_formatted = now.strftime(date_format)
+                if encoding:
+                    date_formatted = date_formatted.decode(encoding)
                 raise Invalid(
                     self.message('future', state,
                                  date=date_formatted),
@@ -855,8 +870,9 @@ class DateValidator(FancyValidator):
             value_as_date = datetime_makedate(
                 dt_mod, value.year, value.month, value.day)
             if value_as_date < today:
-                date_formatted = now.strftime(
-                    self.message('date_format', state))
+                date_formatted = now.strftime(date_format)
+                if encoding:
+                    date_formatted = date_formatted.decode(encoding)
                 raise Invalid(
                     self.message('future', state,
                                  date=date_formatted),
@@ -910,8 +926,20 @@ class Int(FancyValidator):
 
     messages = {
         'integer': _("Please enter an integer value"),
+        'tooLow': _("Please enter an integer above %(min)i"),
+        'tooHigh': _("Please enter an integer below %(max)i"),
         }
 
+    min = None
+    max = None
+    
+    def __initargs__(self, args):
+        if self.min != None:
+            self.min = int(self.min)
+        if self.max != None:
+            self.max = int(self.max)
+
+
     def _to_python(self, value, state):
         try:
             return int(value)
@@ -919,6 +947,14 @@ class Int(FancyValidator):
             raise Invalid(self.message('integer', state),
                           value, state)
 
+    def validate_python(self, value, state):
+        if self.min != None and value < self.min:
+            msg = self.message("tooLow", state, min=self.min)
+            raise Invalid(msg, value, state)
+        if self.max != None and value > self.max:
+            msg = self.message("tooHigh", state, max=self.max)
+            raise Invalid(msg, value, state)
+
     _from_python = _to_python
 
 class Number(FancyValidator):
@@ -962,6 +998,10 @@ class String(FancyValidator):
     Also takes a `max` and `min` argument, and the string length must
     fall in that range.
 
+    Also you may give an `encoding` argument, which will encode any
+    unicode that is found.  Lists and tuples are joined with
+    `list_joiner` (default ``', '``) in ``from_python``.
+
     ::
 
         >>> String(min=2).to_python('a')
@@ -992,6 +1032,8 @@ class String(FancyValidator):
     min = None
     max = None
     not_empty = None
+    encoding = None
+    list_joiner = ', '
 
     messages = {
         'tooLong': _("Enter a value less than %(max)i characters long"),
@@ -1001,8 +1043,37 @@ class String(FancyValidator):
     def __initargs__(self, new_attrs):
         if self.not_empty is None and self.min:
             self.not_empty = True
+
+    def _to_python(self, value, state):
+        if not value:
+            value = ''
+        if not isinstance(value, basestring):
+            try:
+                value = str(value)
+            except UnicodeEncodeError:
+                value = unicode(value)
+        if self.encoding is not None and isinstance(value, unicode):
+            value = value.encode(self.encoding)
+        return value
+
+    def _from_python(self, value, state):
+        if not value and value != 0:
+            value = ''
+        if not isinstance(value, basestring):
+            if isinstance(value, (list, tuple)):
+                value = self.list_joiner.join([
+                    self._from_python(v, state) for v in value])
+            try:
+                value = str(value)
+            except UnicodeEncodeError:
+                value = unicode(value)
+        if self.encoding is not None and isinstance(value, unicode):
+            value = value.encode(self.encoding)
+        if self.strip:
+            value = value.strip()
+        return value
     
-    def validate_python(self, value, state):
+    def validate_other(self, value, state):
         if (self.max is not None and value is not None
             and len(value) > self.max):
             raise Invalid(self.message('tooLong', state,
@@ -1014,13 +1085,6 @@ class String(FancyValidator):
                                        min=self.min),
                           value, state)
 
-    def _from_python(self, value, state):
-        if value:
-            return str(value)
-        if value == 0:
-            return str(value)
-        return ""
-
     def empty_value(self, value):
         return ''
 
@@ -1037,7 +1101,7 @@ class UnicodeString(String):
     ::
     
         >>> UnicodeString().to_python(None)
-        ''
+        u''
         >>> UnicodeString().to_python([])
         u''
         >>> UnicodeString(encoding='utf-7').to_python('Ni Ni Ni')
@@ -1055,23 +1119,35 @@ class UnicodeString(String):
         self.outputEncoding = outputEncoding or self.encoding
     
     def _to_python(self, value, state):
-        if value:
-            if isinstance(value, unicode):
-                return value
+        if not value:
+            return u''
+        if isinstance(value, unicode):
+            return value
+        if not isinstance(value, unicode):
             if hasattr(value, '__unicode__'):
-                return unicode(value)
-            try:
-                return unicode(value, self.inputEncoding)
-            except UnicodeDecodeError:
-                raise Invalid(self.message('badEncoding', state), value, state)
-        return u''
+                value = unicode(value)
+                return value
+            else:
+                value = str(value)
+        try:
+            return unicode(value, self.inputEncoding)
+        except UnicodeDecodeError:
+            raise Invalid(self.message('badEncoding', state), value, state)
+        except TypeError:
+            raise Invalid(self.message('badType', state, type=type(value), value=value), value, state)
     
     def _from_python(self, value, state):
-        if hasattr(value, '__unicode__'):
-            value = unicode(value)
+        if not isinstance(value, unicode):
+            if hasattr(value, '__unicode__'):
+                value = unicode(value)
+            else:
+                value = str(value)
         if isinstance(value, unicode):
-            return value.encode(self.outputEncoding)
-        return str(value)
+            value = value.encode(self.outputEncoding)
+        return value
+
+    def empty_value(self, value):
+        return u''
 
 class Set(FancyValidator):
 
@@ -1104,6 +1180,8 @@ class Set(FancyValidator):
 
     use_set = False
 
+    if_missing = ()
+
     def _to_python(self, value, state):
         if self.use_set:
             if isinstance(value, sets.Set):
@@ -1139,7 +1217,7 @@ class Email(FancyValidator):
     If you pass ``resolve_domain=True``, then it will try to resolve
     the domain name to make sure it's valid.  This takes longer, of
     course.  You must have the `pyDNS <http://pydns.sf.net>`__ modules
-    installed to look up MX records.
+    installed to look up DNS (MX and A) records.
 
     ::
 
@@ -1161,6 +1239,8 @@ class Email(FancyValidator):
         True
         >>> e.to_python('doesnotexist@colorstudy.com')
         'doesnotexist@colorstudy.com'
+        >>> e.to_python('test@forums.nyu.edu')
+        'test@forums.nyu.edu'
         >>> e.to_python('test@thisdomaindoesnotexistithinkforsure.com')
         Traceback (most recent call last):
             ...
@@ -1185,21 +1265,15 @@ class Email(FancyValidator):
         }
     
     def __init__(self, *args, **kw):
-        global mxlookup
         FancyValidator.__init__(self, *args, **kw)
         if self.resolve_domain:
-            if mxlookup is None:
-                try:
-                    import DNS.Base
-                    DNS.Base.ParseResolvConf()
-                    from DNS.lazy import mxlookup
-                except ImportError:
-                    import warnings
-                    warnings.warn(
-                        "pyDNS <http://pydns.sf.net> is not installed on "
-                        "your system (or the DNS package cannot be found).  "
-                        "I cannot resolve domain names in addresses")
-                    raise
+            if not have_dns:
+                import warnings
+                warnings.warn(
+                    "pyDNS <http://pydns.sf.net> is not installed on "
+                    "your system (or the DNS package cannot be found).  "
+                    "I cannot resolve domain names in addresses")
+                raise ImportError, "no module named DNS"
 
     def validate_python(self, value, state):
         if not value:
@@ -1208,38 +1282,37 @@ class Email(FancyValidator):
                 value, state)
         value = value.strip()
         splitted = value.split('@', 1)
-        if not len(splitted) == 2:
+        try:
+            username, domain=splitted
+        except ValueError:
             raise Invalid(
                 self.message('noAt', state),
                 value, state)
-        if not self.usernameRE.search(splitted[0]):
+        if not self.usernameRE.search(username):
             raise Invalid(
                 self.message('badUsername', state,
-                             username=splitted[0]),
+                             username=username),
                 value, state)
-        if not self.domainRE.search(splitted[1]):
+        if not self.domainRE.search(domain):
             raise Invalid(
                 self.message('badDomain', state,
-                             domain=splitted[1]),
+                             domain=domain),
                 value, state)
         if self.resolve_domain:
-	    global socket, DNSError
-	    if socket is None:
-        	import socket
-
-	    if DNSError is None:
-		from DNS.Base import DNSError
-
+            assert have_dns, "pyDNS should be available"
 	    try:
-                domains = mxlookup(splitted[1])
-	    except (socket.error, DNSError), e:
+                a=DNS.DnsRequest(domain, qtype='mx').req().answers
+                if not a:
+                    a=DNS.DnsRequest(domain, qtype='a').req().answers
+                dnsdomains=[x['data'] for x in a]
+	    except (socket.error, DNS.DNSError), e:
 		raise Invalid(
 		    self.message('socketError', state, error=e),
 		    value, state)
-            if not domains:
+            if not dnsdomains:
                 raise Invalid(
                     self.message('domainDoesNotExist', state,
-                                 domain=splitted[1]),
+                                 domain=domain),
                     value, state)
 
     def _to_python(self, value, state):
@@ -1260,9 +1333,7 @@ class URL(FancyValidator):
         >>> u.to_python('foo.com')
         'http://foo.com'
         >>> u.to_python('http://hahaha/bar.html')
-        Traceback (most recent call last):
-            ...
-        Invalid: That is not a valid URL
+        'http://hahaha/bar.html'
         >>> u.to_python('https://test.com')
         'https://test.com'
         >>> u = URL(add_http=False, check_exists=True)
@@ -1424,10 +1495,9 @@ class StateProvince(FancyValidator):
 class PhoneNumber(FancyValidator):
 
     """
-    Validates, and converts to ###-###-####, optionally with
-    extension (as ext.##...)
-    
-    @@: should add international phone number support
+    Validates, and converts to ###-###-####, optionally with extension
+    (as ext.##...).  Only support US phone numbers.  See
+    InternationalPhoneNumber for support for that kind of phone number.
 
     ::
 
@@ -1473,6 +1543,108 @@ class PhoneNumber(FancyValidator):
             result = result + " ext.%s" % match.group(4)
         return result
 
+# from dbmanager.intl_util.validators@r313[208:303]
+class IPhoneNumberValidator(FancyValidator):
+
+    """
+    Validates, and converts phone numbers to +##-###-#######.
+    Adapted from RFC 3966
+
+    ::
+
+        >>> p = IPhoneNumberValidator(default_cc=49)
+        >>> p.to_python('333-3333')
+        Traceback (most recent call last):
+            ...
+        Invalid: Please enter a number, with area code, in the form +##-###-#######.
+        >>> p.to_python('0555/4860-300')
+        '+49-555-4860-300'
+        >>> p.to_python('0555-49924-51')
+        '+49-555-49924-51'
+        >>> p.to_python('0555/8114100')
+        '+49-555-8114100'
+        >>> p.to_python('0555 8114100')
+        '+49-555-8114100'
+        >>> p.to_python(' +49 (0)555 350 60 0')
+        '+49-555-35060-0'
+        >>> p.to_python('+49 555 350600')
+        '+49-555-350600'
+        >>> p.to_python('0049/ 555/ 871 82 96')
+        '+49-555-87182-96'
+        >>> p.to_python('0555-2 50-30')
+        '+49-555-250-30'
+        >>> p.to_python('(05 55)4 94 33 47')
+        '+49-555-49433-47'
+        >>> p.to_python('+973-555431')
+        '+973-555431'
+        >>> p.to_python('1-393-555-3939')
+        '+1-393-555-3939'
+        >>> p.to_python('+43 (1) 55528/0')
+        '+43-1-55528-0'
+        >>> p.to_python('+43 5555 429 62-0')
+        '+43-5555-42962-0'
+        >>> p.to_python('00 218 55 33 50 317 321')
+        '+218-55-3350317-321'
+        >>> p.to_python('+218 (0)55-3636639/38')
+        '+218-55-3636639-38'
+    """
+
+    strip = True
+    # Use if there's a default country code you want to use:
+    default_cc = None
+    _mark_chars_re = re.compile(r"[_.!~*'/]")
+    _preTransformations = [
+        (re.compile(r'^\((\d+)\s*(\d+)\)(.+)$'), '%s%s-%s'),
+        (re.compile(r'^(?:1-)(\d+.+)$'), '+1-%s'),
+        (re.compile(r'^00\s*(\d+.+)$'), '+%s'),
+        (re.compile(r'^(\+\d+)\s+\(0\)\s*(\d+.+)$'), '%s-%s'),
+        (re.compile(r'^(0\d+)\s(\d+)$'), '%s-%s'),
+        ]
+    _ccIncluder = [
+        (re.compile(r'^0([1-9]\d*)\-(\d+.*)$'), '+%d-%s-%s'),
+        ]
+    _postTransformations = [
+        (re.compile(r'^(\+\d+)[-\s]\(?(\d+)\)?[-\s](\d+.+)$'), '%s-%s-%s'),
+        (re.compile(r'^(.+)\s(\d+)$'), '%s-%s'),
+        ]
+    _phoneIsSane = re.compile(r'^(\+[1-9]\d*)-([\d\-]+)$')
+
+    messages = {
+        'phoneFormat': _('Please enter a number, with area code, in the form +##-###-#######.'),
+        }
+
+    def _perform_rex_transformation(self, value, transformations):
+        for rex, trf in transformations:
+            match = rex.search(value)
+            if match:
+                value = trf % match.groups()
+        return value
+
+    def _prepend_country_code(self, value, transformations, country_code):
+        for rex, trf in transformations:
+            match = rex.search(value)
+            if match:
+                return trf % ((country_code,)+match.groups())
+        return value
+
+    def _to_python(self, value, state):
+        self.assert_string(value, state)
+        value = self._perform_rex_transformation(value, self._preTransformations)
+        value = self._mark_chars_re.sub('-', value)
+        if self.default_cc:
+            value = self._prepend_country_code(value, self._ccIncluder, self.default_cc)
+        for f, t in [('  ', ' '), ('--', '-'), (' - ', '-'), ('- ', '-'), (' -', '-')]:
+            value = value.replace(f, t)
+        value = self._perform_rex_transformation(value, self._postTransformations)
+        value = value.replace(' ', '')
+        # did we successfully transform that phone number? Thus, is it valid?
+        if not self._phoneIsSane.search(value):
+            raise Invalid(self.message('phoneFormat', state), value, state)
+        return value
+
+
+
+
 class FieldStorageUploadConverter(FancyValidator):
     """
     Converts a cgi.FieldStorage instance to
@@ -1504,6 +1676,20 @@ class FileUploadKeeper(FancyValidator):
     there may be extra values ``'original_filename'`` and
     ``'original_content'``, which may want to use in your form to show
     the user you still have their content around.
+
+    To use this, make sure you are using variabledecode, then use
+    something like::
+
+      <input type="file" name="myfield.upload">
+      <input type="hidden" name="myfield.static">
+
+    Then in your scheme::
+
+      class MyScheme(Scheme):
+          myfield = FileUploadKeeper()
+
+    Note that big file uploads mean big hidden fields, and lots of
+    bytes passed back and forth in the case of an error.
     """
 
     upload_key = 'upload'
@@ -1622,7 +1808,7 @@ class DateConverter(FancyValidator):
         'invalidDate': _('That is not a valid day (%(exception)s)'),
         'unknownMonthName': _("Unknown month name: %(month)s"),
         'invalidYear': _('Please enter a number for the year'),
-        'fourDigitYear': _('Please enter a four-digit year'),
+        'fourDigitYear': _('Please enter a four-digit year after 1899'),
         'wrongFormat': _('Please enter the date in the form %(format)s'),
         }
 
@@ -1688,7 +1874,7 @@ class DateConverter(FancyValidator):
             year = year + 2000
         if year >= 50 and year < 100:
             year = year + 1900
-        if year > 20 and year < 50:
+        if (year > 20 and year < 50) or (year>99 and year<1900):
             raise Invalid(self.message('fourDigitYear', state),
                           year, state)
         return year
@@ -1717,8 +1903,11 @@ class DateConverter(FancyValidator):
 
     def unconvert_day(self, value, state):
         # @@ ib: double-check, improve
-        return value.strftime("%m/%d/%Y")
-        
+        if self.month_style == 'mm/dd/yyyy':
+            return value.strftime("%m/%d/%Y")
+        else:
+            return value.strftime("%d/%m/%Y")
+
     def unconvert_month(self, value, state):
         # @@ ib: double-check, improve
         return value.strftime("%m/%Y")
@@ -2001,6 +2190,9 @@ class StripField(FancyValidator):
                           valueDict, state)
         return field, v
 
+    def is_empty(self, value):
+        ## Empty dictionaries don't really apply here
+        return False
 
 class StringBool(FancyValidator):
     # Originally from TurboGears
@@ -2111,6 +2303,109 @@ class SignedString(FancyValidator):
             chr(random.randrange(256))
             for i in range(self.nonce_length)])
 
+class CIDR(FancyValidator):
+    """
+    Formencode validator to check whether a string is in correct CIDR
+    notation (IP address, or IP address plus /mask)
+
+    Examples::
+
+        >>> cidr = CIDR()
+        >>> cidr.to_python('127.0.0.1')
+        '127.0.0.1'
+        >>> cidr.to_python('299.0.0.1')
+        Traceback (most recent call last):
+            ...
+        Invalid: The octets must be within the range of 0-255 (not '299')
+        >>> cidr.to_python('192.168.0.1/1')
+        Traceback (most recent call last):
+            ...
+        Invalid: The network size (bits) must be within the range of 8-32 (not '1')
+        >>> cidr.to_python('asdf')
+        Traceback (most recent call last):
+            ...
+        Invalid: Please enter a valid IP address (a.b.c.d) or IP network (a.b.c.d/e)
+    """
+    messages = {
+            'not_cidr_format' : u'Please enter a valid IP address (a.b.c.d) or IP network (a.b.c.d/e)',
+            'illegal_octets' : u'The octets must be within the range of 0-255 (not %(octet)r)',
+            'illegal_bits' : u'The network size (bits) must be within the range of 8-32 (not %(bits)r)',
+            }
+
+    def validate_python(self, value, state):
+        try:
+            # Split into octets and bits
+            if '/' in value: # a.b.c.d/e
+                addr, bits = value.split('/')
+            else: # a.b.c.d
+                addr, bits = value, 32
+                
+            octets = addr.split('.')
+
+            # Only 4 octets?
+            if len(octets) != 4:
+                raise Invalid(self.message("not_cidr_format", state, value=value), value, state)
+
+            # Correct octets?
+            for octet in octets:
+                if int(octet) < 0 or int(octet) > 255:
+                    raise Invalid(self.message("illegal_octets", state, octet=octet), value, state)
+
+            # Bits (netmask) correct?
+            if int(bits) < 8 or int(bits) > 32:
+                    raise Invalid(self.message("illegal_bits", state, bits=bits), value, state)
+
+        # Splitting faild: wrong syntax
+        except ValueError:
+            raise Invalid(self.message("not_cidr_format", state), value, state)
+
+class MACAddress(FancyValidator):
+    """
+    Formencode validator to check whether a string is a correct hardware
+    (MAC) address.
+
+    Examples::
+
+        >>> mac = MACAddress()
+        >>> mac.to_python('aa:bb:cc:dd:ee:ff')
+        'aabbccddeeff'
+        >>> mac.to_python('aa:bb:cc:dd:ee:ff:e')
+        Traceback (most recent call last):
+            ...
+        Invalid: A MAC address must contain 12 digits and A-F; the value you gave has 13 characters
+        >>> mac.to_python('aa:bb:cc:dd:ee:fx')
+        Traceback (most recent call last):
+            ...
+        Invalid: MAC addresses may only contain 0-9 and A-F (and optionally :), not 'x'
+        >>> MACAddress(add_colons=True).to_python('aabbccddeeff')
+        'aa:bb:cc:dd:ee:ff'
+    """
+
+    strip=True
+    valid_characters = '0123456789abcdefABCDEF'
+    add_colons = False
+    
+    messages = {
+        'bad_length': _(u'A MAC address must contain 12 digits and A-F; the value you gave has %(length)s characters'),
+        'bad_character': _(u'MAC addresses may only contain 0-9 and A-F (and optionally :), not %(char)r'),
+        }
+
+    def _to_python(self, value, state):
+        address = value.replace(':','') # remove colons
+        address = address.lower()
+        if len(address)!=12:
+            raise Invalid(self.message("bad_length", state, length=len(address)), address, state)
+        for char in address:
+            if char not in self.valid_characters:
+                raise Invalid(self.message("bad_character", state, char=char), address, state)
+        if self.add_colons:
+            address = '%s:%s:%s:%s:%s:%s' % (
+                address[0:2], address[2:4], address[4:6],
+                address[6:8], address[8:10], address[10:12])
+        return address
+
+    _from_python = _to_python
+
 class FormValidator(FancyValidator):
     """
     A FormValidator is something that can be chained with a
